using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;


namespace ClothingCensor
{
    public class ClothingCensor : MVRScript
    {
		public struct ClothingSelectable
		{
			public DAZSkinWrap skinWrap;
			public JSONStorableBool enabled;
			public Shader[] originalShader;
		}
		protected string ASSETS_BUNDLE_PATH;
		public Dictionary<string, ClothingSelectable> clothingOptions;
		public Shader censor_shader;

		public JSONStorableFloat option_pixelation;

		
		public override void Init()
        {			 
			CreateButton("Rescan active clothes").button.onClick.AddListener(() => {scanClothes();});

			clothingOptions = new Dictionary<string, ClothingSelectable>();
			option_pixelation = new JSONStorableFloat("Pixelation", 6.0f, setPixelation, 1.0f, 1000.0f, true);
			RegisterFloat(option_pixelation);
			CreateSlider(option_pixelation);

			createStaticDescriptionText("Desc1","<color=#000><size=30>Saving and Loading does not work correctly at the moment. You will need to select each at start each time.</size></color>",true,160);
			createStaticDescriptionText("Desc2","<color=#000><size=30>When clothing item is disabled it breaks, you need to Force Reload in the clothing items customize menu.</size></color>",true,120);

            try
            {
				// Start assetbundle load
				ASSETS_BUNDLE_PATH = GetPluginPath(this) + "/assets/cloth_censor.assetbundle";	
				Request request = new AssetLoader.AssetBundleFromFileRequest {path = ASSETS_BUNDLE_PATH, callback = OnAssetBundleLoaded};
				AssetLoader.QueueLoadAssetBundleFromFile(request);
				
			}
            catch(Exception e)
            {
                SuperController.LogError("ClothingCensor - Exception caught: " + e);
            }
		}

		public void setPixelation(float f) {
			updateClothes(false);
		}

		private void scanClothes() {
			DAZClothingItem[] clothing = containingAtom.GetComponentsInChildren<DAZClothingItem>();
			foreach(DAZClothingItem clothing_item in clothing) {
				DAZSkinWrap[] wraps = clothing_item.GetComponentsInChildren<DAZSkinWrap>();
				foreach(DAZSkinWrap wrap in wraps) {
					if(wrap.dazMesh != null && wrap.dazMesh.numMaterials > 0 && !clothingOptions.ContainsKey(wrap.name)){
						ClothingSelectable s = new ClothingSelectable() {skinWrap = wrap};
						s.enabled = new JSONStorableBool(wrap.name, false, updateClothes);
						RegisterBool(s.enabled);
						CreateToggle(s.enabled);
						s.originalShader = new Shader[wrap.dazMesh.numMaterials];
						for(int i=0; i < wrap.dazMesh.numMaterials; ++i) {
							s.originalShader[i] = wrap.dazMesh.materials[i].shader;
						}
						clothingOptions.Add(wrap.name, s);
					}
				}
			}
			updateClothes(false);
		}
				
		private void updateClothes(bool notUsed) {
			foreach(var item in clothingOptions) {
				ClothingSelectable s = item.Value;
				bool changed = false;
				if(s.enabled.val) {
					for(int i=0; i < s.skinWrap.dazMesh.numMaterials; ++i) {
						if(s.skinWrap.dazMesh.materials[i].shader != censor_shader) {
							s.skinWrap.dazMesh.materials[i].shader = censor_shader;
							changed = true;
						}
						s.skinWrap.dazMesh.materials[i].SetFloat ("_Pixelation", option_pixelation.val / 1000.0f);
					}
				} else {
					for(int i=0; i < s.skinWrap.dazMesh.numMaterials; ++i) {
						if(s.skinWrap.dazMesh.materials[i].shader == censor_shader) {
							s.skinWrap.dazMesh.materials[i].shader = s.originalShader[i];
							changed = true;
						}
					}
				}
				if(changed) {
					s.skinWrap.CopyMaterials();
					s.skinWrap.BroadcastMessage("OnApplicationFocus", true);
				}
			}
		}

		private void OnAssetBundleLoaded(Request aRequest) {
			try {
				Shader s = aRequest.assetBundle.LoadAllAssets<Shader>()[0];
				censor_shader = s;
				scanClothes();
			} catch(Exception e) {
				SuperController.LogMessage("Failed to load asset bundle");
			} 
		}

		
		
		private void Start() {
			
		}
		


		void Update() {
			
		}

		public void OnDestroy() {
			foreach(var item in clothingOptions) {
				ClothingSelectable s = item.Value;
				bool changed = false;
				for(int i=0; i < s.skinWrap.dazMesh.numMaterials; ++i) {
					if(s.skinWrap.dazMesh.materials[i].shader == censor_shader) {
						s.skinWrap.dazMesh.materials[i].shader = s.originalShader[i];
						changed = true;
					}
				}
				if(changed) {
					s.skinWrap.CopyMaterials();
					s.skinWrap.BroadcastMessage("OnApplicationFocus", true);
				}
			}
			if(ASSETS_BUNDLE_PATH != null) {
				AssetLoader.DoneWithAssetBundleFromFile(ASSETS_BUNDLE_PATH);
			}
		}
		
		// **************************
		// Functions
		// **************************

		/**** From MacGruber_utils.cs ****/
		// Get directory path where the plugin is located. Based on Alazi's & VAMDeluxe's method.
		public static string GetPluginPath(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			return filename.Substring(0, filename.LastIndexOfAny(new char[] { '/', '\\' }));
		}

		private static void disableScrollOnText(UIDynamicTextField target) {
			ScrollRect targetSR = target.UItext.transform.parent.transform.parent.transform.parent.GetComponent<ScrollRect>();
			if( targetSR != null ) {
				targetSR.horizontal = false;
				targetSR.vertical = false;
			}
		}
		private void createStaticDescriptionText( string DescTitle, string DescText, bool rightSide, int fieldHeight ) {
			JSONStorableString staticDescString = new JSONStorableString(DescTitle,DescText);
			UIDynamicTextField staticDescStringField = CreateTextField(staticDescString, rightSide);
			staticDescStringField.backgroundColor = new Color(1f, 1f, 1f, 0f);
			LayoutElement sdsfLayout = staticDescStringField.GetComponent<LayoutElement>();
			sdsfLayout.preferredHeight = sdsfLayout.minHeight = fieldHeight;
			staticDescStringField.height = fieldHeight;
			disableScrollOnText(staticDescStringField);
		}

	}
}